/*
	Code generation for the following grammar segment:
	
	AllocationExpression :=
		  "new" PrimitiveType() ArrayDimsAndInits()
		| "new" Name() (ArrayDimsAndInits() | (Arguments() [BehaviorBody()])) [BindDeclaration]
		| "reference" Name() Arguments()

*/	
	
package salsa_lite.core.compiler.definitions;

import salsa_lite.core.compiler.SalsaCompiler;
import salsa_lite.core.compiler.SalsaParser;
import salsa_lite.core.compiler.SimpleNode;

public class AllocationExpression extends SimpleNode {

	public AllocationExpression(int id) 			{ super(id); }
	public AllocationExpression(SalsaParser p, int id)	{ super(p, id); }

	public String getType() {
		return getChild(0).getJavaCode();
	}

	public String getJavaCode() {
		if ((getChild(0) instanceof Name) && SalsaCompiler.isActor( getChild(0).getJavaCode() ) && (getChild(1) instanceof Arguments)) {
			String code;
			if (getToken(0).toString().equals("new")) {
				code = ((Name)getChild(0)).getJavaCode() + ".construct(self, ";

				if (getChild( children.length-1 ) instanceof BindDeclaration) {
					BindDeclaration bindDeclaration = (BindDeclaration)getChild( children.length -1);
					code += bindDeclaration.getUAN() + ", ";
					code += bindDeclaration.getHost() + ", ";
					code += bindDeclaration.getPort();
				} else {
					code += "null, null, -1";
				}
				String argsCode = ((Arguments)getChild(1)).getJavaCode();
				code += ", new Object[]{" + argsCode.substring(1, argsCode.length()-1) + "})";
			} else {
				code = "new " + ((Name)getChild(0)).getJavaCode() + "(" + ((Arguments)getChild(1)).getJavaCode() + ")";
			}
			if (System.getProperty("enable_gc") != null) {
				code += ".copy()";
			}
			return code;
		} else {
			return "new " + getChildCode();
		}
	}
}
