/*
	Code generation for the following grammar segment:
	
	BehaviorBody :=
	"{" 
		(
			  Initializer 
			| NestedBehaviorDeclaration 
			| StateVariableDeclaration
			| ConstructorDeclaration
			| MethodDeclaration
		)* 
	"}"
*/

package salsa_lite.core.compiler.definitions;

import salsa_lite.core.compiler.SalsaCompiler;
import salsa_lite.core.compiler.SalsaParser;
import salsa_lite.core.compiler.SimpleNode;
import salsa_lite.core.compiler.SymbolTable;

import java.util.Vector;

public class BehaviorBody extends SimpleNode {
	
	public BehaviorBody(int id) 			{ super(id); }
	public BehaviorBody(SalsaParser p, int id)	{ super(p, id); }

	public String getPreCode()		{
		SalsaCompiler.indent++;
		String actorName = SalsaCompiler.getActorName();

		String code = "{\n";

		String actorModule = SalsaCompiler.getCompilationUnit().getModule();
		String fullActorName = "";
		if (!actorModule.equals("")) fullActorName += actorModule + ".";
		fullActorName += SalsaCompiler.getActorName();

		code += SalsaCompiler.getIndent() + "public " + actorName + "(UAN uan) {\n";
		code += SalsaCompiler.getIndent() + "\tsuper(uan);\n";
		code += SalsaCompiler.getIndent() + "\tself = new " + actorName.substring(0, actorName.length()-5) + "(getUAN());\n";
		code += SalsaCompiler.getIndent() + "}\n\n";

		code += SalsaCompiler.getIndent() + "public " + actorName + "(UAL ual) {\n";
		code += SalsaCompiler.getIndent() + "\tsuper(ual);\n";
		code += SalsaCompiler.getIndent() + "\tself = new " + actorName.substring(0, actorName.length()-5) + "(getUAL());\n";
		code += SalsaCompiler.getIndent() + "}\n\n";

		code += SalsaCompiler.getIndent() + "public " + actorName + "(String identifier) {\n";
		code += SalsaCompiler.getIndent() + "\tsuper(identifier);\n";
		code += SalsaCompiler.getIndent() + "\tself = new " + actorName.substring(0, actorName.length()-5) + "(getUAL());\n";
		code += SalsaCompiler.getIndent() + "}\n\n";

		if (!SymbolTable.isAbstract) {
			code += SalsaCompiler.getIndent() + "public Object invokeMessage(String methodName, Object[] arguments) throws CurrentContinuationException, MessageHandlerNotFoundException, Exception {\n";

			Vector methodNames = new Vector();
			Vector parameters = new Vector();
			Vector constructorParameters = new Vector();

			boolean hasConstructor = false;
			if (children != null) {
				code += SalsaCompiler.getIndent() + "\t";

				for (int i = 0; i < children.length; i++) {
					if (getChild(i) instanceof MethodDeclaration) {
						String methodName = ((MethodDeclaration)getChild(i)).getMethodName();
						String[] parameterTypes = ((MethodDeclaration)getChild(i)).getParameterTypes();
						if (parameterTypes == null) parameterTypes = new String[0];
						String returnType = ((MethodDeclaration)getChild(i)).getReturnType();

						code += "if (methodName.equals(\"" + methodName + "\") && arguments.length == " + parameterTypes.length;
						for (int j = 0; j < parameterTypes.length; j++) {
							code += " && (arguments[" + j + "] == null || arguments[" + j + "] instanceof ";
							if (parameterTypes[j].equals("int")) {
								code += "Integer";
							} else if (parameterTypes[j].equals("double")) {
								code += "Double";
							} else if (parameterTypes[j].equals("long")) {
								code += "Long";
							} else if (parameterTypes[j].equals("float")) {
								code += "Float";
							} else if (parameterTypes[j].equals("char")) {
								code += "Character";
							} else if (parameterTypes[j].equals("byte")) {
								code += "Byte";
							} else if (parameterTypes[j].equals("short")) {
								code += "Short";
							} else if (parameterTypes[j].equals("boolean")) {
								code += "Boolean";
							} else {
								code += parameterTypes[j];
							}
							code += ")";
						}
						code += ") {\n";

						boolean returnedPrimitive = false;
						if (returnType.equals("void")) {
							code += SalsaCompiler.getIndent() + "\t\t" + methodName + "(";
						} else {
							code += SalsaCompiler.getIndent() + "\t\treturn ";
							if (returnType.equals("int")) {
								returnedPrimitive = true;
								code += "new Integer(";
							} else if (returnType.equals("double")) {
								returnedPrimitive = true;
								code += "new Double(";
							} else if (returnType.equals("long")) {
								returnedPrimitive = true;
								code += "new Long(";
							} else if (returnType.equals("float")) {
								returnedPrimitive = true;
								code += "new Float(";
							} else if (returnType.equals("char")) {
								returnedPrimitive = true;
								code += "new Character(";
							} else if (returnType.equals("byte")) {
								returnedPrimitive = true;
								code += "new Byte(";
							} else if (returnType.equals("short")) {
								returnedPrimitive = true;
								code += "new Short(";
							} else if (returnType.equals("boolean")) {
								returnedPrimitive = true;
								code += "new Boolean(";
							}
							code += methodName + "(";
						}
						for (int j = 0; j < parameterTypes.length; j++) {
							if (parameterTypes[j].equals("int")) {
								code += "((Integer)arguments[" + j + "]).intValue()";
							} else if (parameterTypes[j].equals("double")) {
								code += "((Double)arguments[" + j + "]).doubleValue()";
							} else if (parameterTypes[j].equals("long")) {
								code += "((Long)arguments[" + j + "]).longValue()";
							} else if (parameterTypes[j].equals("float")) {
								code += "((Float)arguments[" + j + "]).floatValue()";
							} else if (parameterTypes[j].equals("char")) {
								code += "((Character)arguments[" + j + "]).charValue()";
							} else if (parameterTypes[j].equals("byte")) {
								code += "((Byte)arguments[" + j + "]).byteValue()";
							} else if (parameterTypes[j].equals("short")) {
								code += "((Short)arguments[" + j + "]).shortValue()";
							} else if (parameterTypes[j].equals("boolean")) {
								code += "((Boolean)arguments[" + j + "]).booleanValue()";
							} else {
								code += "(" + parameterTypes[j] + ")arguments[" + j + "]";
							}
							if (j != parameterTypes.length-1) code += ", ";
						}
						if (returnedPrimitive) code += ")";
						code += ");\n";
						if (returnType.equals("void")) code += SalsaCompiler.getIndent() + "\t\treturn null;\n";
						code += SalsaCompiler.getIndent() + "\t}  else ";

					} else if (getChild(i) instanceof ConstructorDeclaration) {
						hasConstructor = true;
						String[] parameterTypes = ((ConstructorDeclaration)getChild(i)).getParameterTypes();
						if (parameterTypes == null) parameterTypes = new String[0];

						code += "if (methodName.equals(\"construct\") && arguments.length == " + parameterTypes.length;
						for (int j = 0; j < parameterTypes.length; j++) {
							code += " && (arguments[" + j + "] == null || arguments[" + j + "] instanceof ";
							if (parameterTypes[j].equals("int")) {
								code += "Integer";
							} else if (parameterTypes[j].equals("double")) {
								code += "Double";
							} else if (parameterTypes[j].equals("long")) {
								code += "Long";
							} else if (parameterTypes[j].equals("float")) {
								code += "Float";
							} else if (parameterTypes[j].equals("char")) {
								code += "Character";
							} else if (parameterTypes[j].equals("byte")) {
								code += "Byte";
							} else if (parameterTypes[j].equals("short")) {
								code += "Short";
							} else if (parameterTypes[j].equals("boolean")) {
								code += "Boolean";
							} else {
								code += parameterTypes[j];
							}
							code += ")";
						}
						code += ") {\n";


						code += SalsaCompiler.getIndent() + "\t\tconstruct(";
						for (int j = 0; j < parameterTypes.length; j++) {
							if (parameterTypes[j].equals("int")) {
								code += "((Integer)arguments[" + j + "]).intValue()";
							} else if (parameterTypes[j].equals("double")) {
								code += "((Double)arguments[" + j + "]).doubleValue()";
							} else if (parameterTypes[j].equals("long")) {
								code += "((Long)arguments[" + j + "]).longValue()";
							} else if (parameterTypes[j].equals("float")) {
								code += "((Float)arguments[" + j + "]).floatValue()";
							} else if (parameterTypes[j].equals("char")) {
								code += "((Character)arguments[" + j + "]).charValue()";
							} else if (parameterTypes[j].equals("byte")) {
								code += "((Byte)arguments[" + j + "]).byteValue()";
							} else if (parameterTypes[j].equals("short")) {
								code += "((Short)arguments[" + j + "]).shortValue()";
							} else if (parameterTypes[j].equals("boolean")) {
								code += "((Boolean)arguments[" + j + "]).booleanValue()";
							} else {
								code += "(" + parameterTypes[j] + ")arguments[" + j + "]";
							}
							if (j != parameterTypes.length-1) code += ", ";
						}
						code += ");\n";
						code += SalsaCompiler.getIndent() + "\t\treturn null;\n";
						code += SalsaCompiler.getIndent() + "\t}  else ";
					}
				}

				if (!hasConstructor) {
					code += "if (methodName.equals(\"construct\") && arguments.length == 0) {\n";
					code += SalsaCompiler.getIndent() + "\t\tconstruct();\n";
					code += SalsaCompiler.getIndent() + "\t\treturn null;\n";
					code += SalsaCompiler.getIndent() + "\t} else ";
				}
			}

			code += "{\n";
			code += SalsaCompiler.getIndent() + "\t\treturn super.invokeMessage(methodName, arguments);\n";
			code += SalsaCompiler.getIndent() + "\t}\n";
			code += SalsaCompiler.getIndent() + "}\n\n";

			if (!hasConstructor) {
				code += SalsaCompiler.getIndent() + "void construct() {}\n\n";
			}
		}

		return code;
	}

	public String getPostCode() {
		SalsaCompiler.indent--;

		String code = "}";

		SymbolTable.isAbstract = false;
		return code;
	}

	public String getChildCode() {
		String code = "";

		if (children != null) {		
			for (int i = 0; i < children.length; i++) {
				code += SalsaCompiler.getIndent() + getChild(i).getJavaCode();
			}
		}
		
		return code;
	}
}
