package salsa_lite.core.language;

import salsa_lite.core.naming.UAL;
import salsa_lite.core.services.ServiceFactory;

public class IdentifierGenerator {

	private static String localhost;
	private static int localport;

	private static int number_generators = 50;
	private static Object[] locks;
	private static int[] id_generators;

	static {
		IdentifierGenerator.localhost = ServiceFactory.getReception().getLocalHost();
		IdentifierGenerator.localport = ServiceFactory.getReception().getLocalPort();

		if (System.getProperty("nidgens") != null) {
			IdentifierGenerator.number_generators = Integer.parseInt( System.getProperty("nidgens") );
		}
		IdentifierGenerator.locks = new Object[IdentifierGenerator.number_generators];
		IdentifierGenerator.id_generators = new int[IdentifierGenerator.number_generators];
		for (int i = 0; i < IdentifierGenerator.number_generators; i++) {
			IdentifierGenerator.locks[i] = new Object();
			IdentifierGenerator.id_generators[i] = 0;
		}
	}

	public static UAL generateUniqueUAL(String host, int port) {
		if (host == null) host = localhost;
		if (port <= 0) port = localport;
		String retval = "rmsp://" + host + ":" + port + "/" + host + "/" + port + "/" + System.currentTimeMillis() + "/gen_";

		int target = (int)(Math.random() * IdentifierGenerator.number_generators);
		retval += target + "/actor_";

		Object lock = IdentifierGenerator.locks[ target ];
		synchronized (lock) {
			retval += IdentifierGenerator.id_generators[target];
			IdentifierGenerator.id_generators[target]++;
		}

		UAL ual = null;
		try {
			ual = new UAL(retval);
		} catch (Exception e) {
			System.err.println("IdentifierGenerator Exception generating new UAL: " + e);
			e.printStackTrace();
		}
		return ual;
	}

	public static String generateUniqueMessageId() {
		String retval = localhost + ":" + localport + "/" + System.currentTimeMillis() + "/gen_";

		int target = (int)(Math.random() * IdentifierGenerator.number_generators);
		retval += target + "/mid_";

		Object lock = IdentifierGenerator.locks[ target ];
		synchronized (lock) {
			retval += IdentifierGenerator.id_generators[target];
			IdentifierGenerator.id_generators[target]++;
		}

		return retval;
	}

}
