package salsa_lite.examples.matrixmultiply;

// Import declarations generated by the SALSA compiler, do not modify.
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import salsa_lite.core.language.Actor;
import salsa_lite.core.language.ActorReference;
import salsa_lite.core.language.IdentifierGenerator;
import salsa_lite.core.language.Message;
import salsa_lite.core.language.Token;
import salsa_lite.core.language.exceptions.CurrentContinuationException;
import salsa_lite.core.language.exceptions.MessageHandlerNotFoundException;

import salsa_lite.core.language.CollectibleActorState;

import salsa_lite.core.naming.UAN;
import salsa_lite.core.naming.UAL;
import salsa_lite.core.naming.MalformedUALException;
import salsa_lite.core.naming.MalformedUANException;

// End SALSA compiler generated import delcarations.


public class MatrixMultiplyState extends CollectibleActorState  {
	public MatrixMultiplyState(UAN uan) {
		super(uan);
		self = new MatrixMultiply(getUAN());
	}

	public MatrixMultiplyState(UAL ual) {
		super(ual);
		self = new MatrixMultiply(getUAL());
	}

	public MatrixMultiplyState(String identifier) {
		super(identifier);
		self = new MatrixMultiply(getUAL());
	}

	public Object invokeMessage(String methodName, Object[] arguments) throws CurrentContinuationException, MessageHandlerNotFoundException, Exception {
		if (methodName.equals("construct") && arguments.length == 0) {
			construct();
			return null;
		}  else if (methodName.equals("act") && arguments.length == 1 && (arguments[0] == null || arguments[0] instanceof String[])) {
			act((String[])arguments[0]);
			return null;
		}  else if (methodName.equals("finished") && arguments.length == 0) {
			finished();
			return null;
		}  else if (methodName.equals("getSection") && arguments.length == 5 && (arguments[0] == null || arguments[0] instanceof double[][]) && (arguments[1] == null || arguments[1] instanceof Integer) && (arguments[2] == null || arguments[2] instanceof Integer) && (arguments[3] == null || arguments[3] instanceof Integer) && (arguments[4] == null || arguments[4] instanceof Integer)) {
			return getSection((double[][])arguments[0], ((Integer)arguments[1]).intValue(), ((Integer)arguments[2]).intValue(), ((Integer)arguments[3]).intValue(), ((Integer)arguments[4]).intValue());
		}  else if (methodName.equals("matrix_multiply") && arguments.length == 3 && (arguments[0] == null || arguments[0] instanceof Integer) && (arguments[1] == null || arguments[1] instanceof double[][]) && (arguments[2] == null || arguments[2] instanceof double[][])) {
			return matrix_multiply(((Integer)arguments[0]).intValue(), (double[][])arguments[1], (double[][])arguments[2]);
		}  else if (methodName.equals("printMatrix") && arguments.length == 1 && (arguments[0] == null || arguments[0] instanceof double[][])) {
			printMatrix((double[][])arguments[0]);
			return null;
		}  else if (methodName.equals("add") && arguments.length == 2 && (arguments[0] == null || arguments[0] instanceof double[][]) && (arguments[1] == null || arguments[1] instanceof double[][])) {
			return add((double[][])arguments[0], (double[][])arguments[1]);
		}  else if (methodName.equals("multiply") && arguments.length == 2 && (arguments[0] == null || arguments[0] instanceof double[][]) && (arguments[1] == null || arguments[1] instanceof double[][])) {
			return multiply((double[][])arguments[0], (double[][])arguments[1]);
		}  else if (methodName.equals("compose") && arguments.length == 4 && (arguments[0] == null || arguments[0] instanceof double[][]) && (arguments[1] == null || arguments[1] instanceof double[][]) && (arguments[2] == null || arguments[2] instanceof double[][]) && (arguments[3] == null || arguments[3] instanceof double[][])) {
			return compose((double[][])arguments[0], (double[][])arguments[1], (double[][])arguments[2], (double[][])arguments[3]);
		}  else {
			return super.invokeMessage(methodName, arguments);
		}
	}

	public  long time;
	public void construct(){
	}
	public void act(String[] arguments) {
		if (arguments.length!=2) {
			System.err.println("Usage: java examples.matrixmultiply.MatrixMultiply <matrix_size> <recursion_depth>");
			System.err.println("\tNote: matrix_size must be a power of 2: 1024, 2048, 4096, 8192, 16384...2^N");
			System.err.println("\tNote: will create 8^recursion_depth actors.");
			System.exit(0);
		}
		int matrix_size = Integer.parseInt(arguments[0]);
		int recursion_depth = Integer.parseInt(arguments[1]);
		int number_actors = (int)Math.pow(8, recursion_depth);
		double[][] A = new double[matrix_size][matrix_size];
		for (int i = 0; i<matrix_size; i++){
			for (int j = 0; j<matrix_size; j++){
				A[i][j] = Math.random();
			}
		}
		double[][] B = new double[matrix_size][matrix_size];
		for (int i = 0; i<matrix_size; i++){
			for (int j = 0; j<matrix_size; j++){
				B[i][j] = Math.random();
			}
		}
		System.out.println("Number of actors: "+number_actors);
		System.out.println("Matrix size: "+matrix_size+" x "+matrix_size);
		Token C = new Token("C");
		{
			// token C = matrix_multiply(new Integer(recursion_depth), A, B)
			{
				Message __message = new Message( self, self.copy(), "matrix_multiply", new Object[]{ new Integer(recursion_depth), A, B }, null, C);
				__messages.add( __message );
			}
			// finished()
			{
				Message __message = new Message( self, self.copy(), "finished", new Object[]{  }, C, null);
				__messages.add( __message );
			}
		}
		time = System.currentTimeMillis();
	}
	public void finished() {
		System.err.println("Multiplied in: "+(System.currentTimeMillis()-time));
	}
	public double[][] getSection(double[][] matrix, int x, int x_max, int y, int y_max) {
		int x_length = x_max-x;
		int y_length = y_max-y;
		double[][] result = new double[x_length][y_length];
		for (int i = 0; i<x_length; i++){
			for (int j = 0; j<y_length; j++){
				result[i][j] = matrix[x+i][x+j];
			}
		}
		return result;
	}
	public double[][] matrix_multiply(int depth, double[][] A, double[][] B) {
		if (A.length!=A[0].length) {
			{
				Token token_3_0 = new Token();
				// standardOutput<-println("Matrix A not square: ")
				{
					Message __message = new Message( self, standardOutput, "println", new Object[]{ "Matrix A not square: " }, null, token_3_0);
					__messages.add( __message );
				}
				// printMatrix(A)
				{
					Message __message = new Message( self, self.copy(), "printMatrix", new Object[]{ A }, token_3_0, null);
					__messages.add( __message );
				}
			}
		}
		if (B.length!=B[0].length) {
			{
				Token token_3_0 = new Token();
				// standardOutput<-println("Matrix B not square: ")
				{
					Message __message = new Message( self, standardOutput, "println", new Object[]{ "Matrix B not square: " }, null, token_3_0);
					__messages.add( __message );
				}
				// printMatrix(B)
				{
					Message __message = new Message( self, self.copy(), "printMatrix", new Object[]{ B }, token_3_0, null);
					__messages.add( __message );
				}
			}
		}
		if ((A.length%2!=0)||(A[0].length%2!=0)) {
			{
				Token token_3_0 = new Token();
				// standardOutput<-println("Matrix A not even: ")
				{
					Message __message = new Message( self, standardOutput, "println", new Object[]{ "Matrix A not even: " }, null, token_3_0);
					__messages.add( __message );
				}
				// printMatrix(A)
				{
					Message __message = new Message( self, self.copy(), "printMatrix", new Object[]{ A }, token_3_0, null);
					__messages.add( __message );
				}
			}
		}
		if ((B.length%2!=0)||(B[0].length%2!=0)) {
			{
				Token token_3_0 = new Token();
				// standardOutput<-println("Matrix B not even: ")
				{
					Message __message = new Message( self, standardOutput, "println", new Object[]{ "Matrix B not even: " }, null, token_3_0);
					__messages.add( __message );
				}
				// printMatrix(B)
				{
					Message __message = new Message( self, self.copy(), "printMatrix", new Object[]{ B }, token_3_0, null);
					__messages.add( __message );
				}
			}
		}
		if (depth==0) 		{
			// multiply(A, B)
			{
				Message __message = new Message( self, self.copy(), "multiply", new Object[]{ A, B }, null, currentMessage.getContinuationToken());
				__message.setJoinPosition(currentMessage.getJoinPosition());
				__messages.add( __message );
			}
			throw new CurrentContinuationException();
		}
		else {
			depth--;
			MatrixMultiply[] workers = new MatrixMultiply[8];
			for (int i = 0; i<8; i++){
				workers[i] = MatrixMultiply.construct(self, null, null, -1, new Object[]{}).copy();
			}
			Token a = new Token("a");
			{
				// token a = getSection(A, new Integer(0), new Integer(A.length/2), new Integer(0), new Integer(A.length/2))
				{
					Message __message = new Message( self, self.copy(), "getSection", new Object[]{ A, new Integer(0), new Integer(A.length/2), new Integer(0), new Integer(A.length/2) }, null, a);
					__messages.add( __message );
				}
			}
			Token b = new Token("b");
			{
				// token b = getSection(A, new Integer(A.length/2), new Integer(A.length), new Integer(0), new Integer(A.length/2))
				{
					Message __message = new Message( self, self.copy(), "getSection", new Object[]{ A, new Integer(A.length/2), new Integer(A.length), new Integer(0), new Integer(A.length/2) }, null, b);
					__messages.add( __message );
				}
			}
			Token c = new Token("c");
			{
				// token c = getSection(A, new Integer(0), new Integer(A.length/2), new Integer(A.length/2), new Integer(A.length))
				{
					Message __message = new Message( self, self.copy(), "getSection", new Object[]{ A, new Integer(0), new Integer(A.length/2), new Integer(A.length/2), new Integer(A.length) }, null, c);
					__messages.add( __message );
				}
			}
			Token d = new Token("d");
			{
				// token d = getSection(A, new Integer(A.length/2), new Integer(A.length), new Integer(A.length/2), new Integer(A.length))
				{
					Message __message = new Message( self, self.copy(), "getSection", new Object[]{ A, new Integer(A.length/2), new Integer(A.length), new Integer(A.length/2), new Integer(A.length) }, null, d);
					__messages.add( __message );
				}
			}
			Token e = new Token("e");
			{
				// token e = getSection(B, new Integer(0), new Integer(A.length/2), new Integer(0), new Integer(A.length/2))
				{
					Message __message = new Message( self, self.copy(), "getSection", new Object[]{ B, new Integer(0), new Integer(A.length/2), new Integer(0), new Integer(A.length/2) }, null, e);
					__messages.add( __message );
				}
			}
			Token f = new Token("f");
			{
				// token f = getSection(B, new Integer(A.length/2), new Integer(A.length), new Integer(0), new Integer(A.length/2))
				{
					Message __message = new Message( self, self.copy(), "getSection", new Object[]{ B, new Integer(A.length/2), new Integer(A.length), new Integer(0), new Integer(A.length/2) }, null, f);
					__messages.add( __message );
				}
			}
			Token g = new Token("g");
			{
				// token g = getSection(B, new Integer(0), new Integer(A.length/2), new Integer(A.length/2), new Integer(A.length))
				{
					Message __message = new Message( self, self.copy(), "getSection", new Object[]{ B, new Integer(0), new Integer(A.length/2), new Integer(A.length/2), new Integer(A.length) }, null, g);
					__messages.add( __message );
				}
			}
			Token h = new Token("h");
			{
				// token h = getSection(B, new Integer(A.length/2), new Integer(A.length), new Integer(A.length/2), new Integer(A.length))
				{
					Message __message = new Message( self, self.copy(), "getSection", new Object[]{ B, new Integer(A.length/2), new Integer(A.length), new Integer(A.length/2), new Integer(A.length) }, null, h);
					__messages.add( __message );
				}
			}
			Token ae = new Token("ae");
			{
				// token ae = workers[0]<-multiply(a, e)
				{
					Message __message = new Message( self, workers[0], "multiply", new Object[]{ a, e }, null, ae);
					__messages.add( __message );
				}
			}
			Token bg = new Token("bg");
			{
				// token bg = workers[1]<-multiply(b, g)
				{
					Message __message = new Message( self, workers[1], "multiply", new Object[]{ b, g }, null, bg);
					__messages.add( __message );
				}
			}
			Token r = new Token("r");
			{
				// token r = workers[1]<-add(ae, bg)
				{
					Message __message = new Message( self, workers[1], "add", new Object[]{ ae, bg }, null, r);
					__messages.add( __message );
				}
			}
			Token af = new Token("af");
			{
				// token af = workers[2]<-multiply(a, f)
				{
					Message __message = new Message( self, workers[2], "multiply", new Object[]{ a, f }, null, af);
					__messages.add( __message );
				}
			}
			Token bh = new Token("bh");
			{
				// token bh = workers[3]<-multiply(b, h)
				{
					Message __message = new Message( self, workers[3], "multiply", new Object[]{ b, h }, null, bh);
					__messages.add( __message );
				}
			}
			Token s = new Token("s");
			{
				// token s = workers[3]<-add(af, bh)
				{
					Message __message = new Message( self, workers[3], "add", new Object[]{ af, bh }, null, s);
					__messages.add( __message );
				}
			}
			Token ce = new Token("ce");
			{
				// token ce = workers[4]<-multiply(c, e)
				{
					Message __message = new Message( self, workers[4], "multiply", new Object[]{ c, e }, null, ce);
					__messages.add( __message );
				}
			}
			Token cg = new Token("cg");
			{
				// token cg = workers[5]<-multiply(c, g)
				{
					Message __message = new Message( self, workers[5], "multiply", new Object[]{ c, g }, null, cg);
					__messages.add( __message );
				}
			}
			Token t = new Token("t");
			{
				// token t = workers[5]<-add(cg, ce)
				{
					Message __message = new Message( self, workers[5], "add", new Object[]{ cg, ce }, null, t);
					__messages.add( __message );
				}
			}
			Token cf = new Token("cf");
			{
				// token cf = workers[6]<-multiply(c, f)
				{
					Message __message = new Message( self, workers[6], "multiply", new Object[]{ c, f }, null, cf);
					__messages.add( __message );
				}
			}
			Token dh = new Token("dh");
			{
				// token dh = workers[7]<-multiply(d, h)
				{
					Message __message = new Message( self, workers[7], "multiply", new Object[]{ d, h }, null, dh);
					__messages.add( __message );
				}
			}
			Token u = new Token("u");
			{
				// token u = workers[7]<-add(cf, dh)
				{
					Message __message = new Message( self, workers[7], "add", new Object[]{ cf, dh }, null, u);
					__messages.add( __message );
				}
			}
			{
				// compose(r, s, t, u)
				{
					Message __message = new Message( self, self.copy(), "compose", new Object[]{ r, s, t, u }, null, currentMessage.getContinuationToken());
					__message.setJoinPosition(currentMessage.getJoinPosition());
					__messages.add( __message );
				}
				throw new CurrentContinuationException();
			}
		}
	}
	public void printMatrix(double[][] m) {
		System.err.println("Matrix:");
		for (int i = 0; i<m.length; i++){
			System.err.print("[ ");
			for (int j = 0; j<m.length; j++){
				System.err.print(m[i][j]+" ");
			}
			System.err.println("]");
		}
		{
			// standardOutput<-println()
			{
				Message __message = new Message( self, standardOutput, "println", new Object[]{  }, null, null);
				__messages.add( __message );
			}
		}
	}
	public double[][] add(double[][] a, double[][] b) {
		double[][] result = new double[a.length][a.length];
		for (int i = 0; i<a.length; i++){
			for (int j = 0; j<a.length; j++){
				result[i][j] = a[i][j]+b[i][j];
			}
		}
		return result;
	}
	public double[][] multiply(double[][] a, double[][] b) {
		double[][] result = new double[a.length][a.length];
		for (int i = 0; i<a.length; i++){
			for (int j = 0; j<a.length; j++){
				result[i][j] = 0;
				for (int k = 0; k<a.length; k++){
					result[i][j] += a[i][k]*b[k][j];
				}
			}
		}
		return result;
	}
	public double[][] compose(double[][] r, double[][] s, double[][] t, double[][] u) {
		double[][] result = new double[r.length*2][r.length*2];
		for (int i = 0; i<r.length; i++){
			for (int j = 0; j<r.length; j++){
				result[i][j] = r[i][j];
			}
		}
		for (int i = r.length; i<2*r.length; i++){
			for (int j = 0; j<r.length; j++){
				result[i][j] = s[i-r.length][j];
			}
		}
		for (int i = 0; i<r.length; i++){
			for (int j = r.length; j<2*r.length; j++){
				result[i][j] = t[i][j-r.length];
			}
		}
		for (int i = r.length; i<2*r.length; i++){
			for (int j = r.length; j<2*r.length; j++){
				result[i][j] = r[i-r.length][j-r.length];
			}
		}
		return result;
	}
}