package salsa_lite.local_gc;

import salsa_lite.core.services.GarbageService;
import salsa_lite.core.services.ServiceFactory;

import salsa_lite.core.language.ActorReference;

import java.util.Hashtable;
import java.util.Vector;

public class LocalGarbageService implements GarbageService {
	int number_collectors = 1;
	CollectionActor[] collectors;

	public LocalGarbageService() {
		System.err.println("created garbage service");

		if (System.getProperty("ncollectors") != null) {
			number_collectors = Integer.parseInt( System.getProperty("ncollectors") );
		}

		collectors = new CollectionActor[number_collectors];
		for (int i = 0; i < number_collectors; i++) {
			collectors[i] = new CollectionActor(i);
//			collectors[i].start();
			System.err.println("created garbage collector[" + i + "]");
		}

		Runnable gcThread = new Runnable() {
			public void run() {
				while (true) {
					System.gc();
					synchronized(this) {
						try {
							wait(2000);
						} catch (Exception e) {
							System.err.println("EXCEPTION ON GC THREAD: " + e);
						}
					}
				}
			}
		};
		new Thread(gcThread, "GC THREAD").start();
	}

	/********
		*	Increment the reference weight of an actor
	 ********/
	public void registerReference(String id, int hashcode, ActorReference reference) {
		collectors[getTarget(hashcode)].register(id, reference);
	}

	/********
		*	Increment the reference weight of an actor
	 ********/
	public void incrementReference(String id, int hashcode, long weight) {
		collectors[getTarget(hashcode)].increment(id, weight);
	}


	/********
		*	Decremenet the reference weight of an actor
	 ********/
	public void decrementReference(String id, int hashcode, long weight) {
		collectors[getTarget(hashcode)].decrement(id, weight);
	}



	/********
		*	Hash function to determine which stage an actor is on
	 ********/
	public int getTarget(int hashcode) {
		return Math.abs(hashcode % number_collectors);
	}
}
