package salsa_lite.wwc.transport;

import java.util.Hashtable;
import java.util.Vector;

import salsa_lite.core.services.ServiceFactory;

public class TransportActor extends Thread {

	private int id;
	private Vector<SendObjectMessage> mailbox;
	private Hashtable<String, OutgoingSocketHandler> handlers;

	protected TransportActor(int id) {
		this.id = id;

		mailbox = new Vector<SendObjectMessage>();
		handlers = new Hashtable<String, OutgoingSocketHandler>();
	}

	public void run() {
		setName("TransportActor_" + id);

		while (true) {
			SendObjectMessage current = getNext();

			String targetHost = current.getTargetHost();
			int targetPort = current.getTargetPort();

			if (ServiceFactory.getReception().isLocal(targetHost, targetPort)) {
				System.err.println("ERROR SENDING OBJECT: " + current.getObject());
				System.err.println("targetHost and targetPort are local: " + targetHost + ":" + targetPort);
			}

			String targetId = targetHost + ":" + targetPort;

			OutgoingSocketHandler outgoing = handlers.get(targetId);
			if (outgoing == null) {
				outgoing = new OutgoingSocketHandler(targetHost, targetPort);
				handlers.put(targetId, outgoing);
			}
			outgoing.send(current.getObject());
		}
	}

	protected synchronized void put(SendObjectMessage som) {
		mailbox.add(som);
		notify();
	}

	private synchronized SendObjectMessage getNext() {
		if (mailbox.isEmpty()) {
			try {
				wait();
			} catch (Exception e) {
				System.err.println("TransportActor error.  waiting on message.");
				System.err.println("Exception: " + e);
				e.printStackTrace();
			}
		}
		return mailbox.remove(0);
	}
}
